<?php
/**
 * Plugin Name: Watermark Text
 * Description: Add text watermarks to uploaded images. Supports .jpg, .png, and .webp formats. Allows watermark customization through settings.
 * Version: 1.5
 * Author: nakhonitech.com
 * Author URI: https://www.nakhonitech.com/
 * License: GPLv2 or later
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 * Requires at least: 5.0
 * Tested up to: 6.7
 * Requires PHP: 7.0
 * Tags: watermark, image, media
 *
 * Copyright © 2024 nakhonitech.com. All rights reserved.
 */

// ตรวจสอบว่าโฟลเดอร์ปลั๊กอินมีชื่อที่ถูกต้อง
add_action('admin_init', 'validate_plugin_folder');

function validate_plugin_folder() {
    $current_plugin_path = plugin_basename(__FILE__);
    $expected_path = 'watermark-text/watermark-text-plugin.php'; // ชื่อโฟลเดอร์และไฟล์ปลั๊กอิน

    if ($current_plugin_path !== $expected_path) {
        deactivate_plugins(plugin_basename(__FILE__)); // ปิดการใช้งานปลั๊กอิน
        wp_die(
            'ปลั๊กอินนี้ถูกแก้ไขชื่อไฟล์หรือโฟลเดอร์ การทำงานถูกหยุดเพื่อป้องกันลิขสิทธิ์',
            'ปลั๊กอินหยุดทำงาน',
            ['back_link' => true]
        );
    }
}

// เพิ่มเมนูหลักและเมนูย่อย
add_action('admin_menu', 'watermark_main_menu');

function watermark_main_menu() {
    add_menu_page(
        'Watermark Settings', 
        'Watermark', 
        'manage_options', 
        'watermark-settings', 
        'watermark_settings_page', 
        'dashicons-format-image', 
        25
    );

    add_submenu_page(
        'watermark-settings', 
        'Apply Watermark', 
        'Apply Watermark', 
        'manage_options', 
        'apply-watermark', 
        'watermark_bulk_apply_page'
    );
}

// Settings Page
function watermark_settings_page() {
    if ($_SERVER['REQUEST_METHOD'] == 'POST') {
        if (isset($_POST['watermark_text'])) {
            update_option('watermark_text', sanitize_text_field($_POST['watermark_text']));
        }
        if (isset($_POST['watermark_font_size'])) {
            update_option('watermark_font_size', intval($_POST['watermark_font_size']));
        }
        if (isset($_POST['watermark_color'])) {
            update_option('watermark_color', sanitize_text_field($_POST['watermark_color']));
        }
        echo '<div class="updated"><p>Settings saved.</p></div>';
    }

    $watermark_text = get_option('watermark_text', 'Your Watermark Text');
    $watermark_font_size = get_option('watermark_font_size', 16);
    $watermark_color = get_option('watermark_color', '#ffffff');

    ?>
    <div class="wrap">
        <h1>Watermark Settings</h1>
        <form method="POST">
            <table class="form-table">
                <tr valign="top">
                    <th scope="row">Watermark Text</th>
                    <td>
                        <input type="text" name="watermark_text" value="<?php echo esc_attr($watermark_text); ?>" style="width: 300px;" />
                    </td>
                </tr>
                <tr valign="top">
                    <th scope="row">Font Size</th>
                    <td>
                        <input type="number" name="watermark_font_size" value="<?php echo esc_attr($watermark_font_size); ?>" style="width: 80px;" /> px
                    </td>
                </tr>
                <tr valign="top">
                    <th scope="row">Font Color</th>
                    <td>
                        <input type="color" name="watermark_color" value="<?php echo esc_attr($watermark_color); ?>" />
                    </td>
                </tr>
            </table>
            <?php submit_button('Save Settings'); ?>
        </form>
        <hr>
        <p style="text-align: center; font-size: 14px; color: gray;">
            Powered by <strong>nakhonitech.com</strong>. แจกจ่ายเพื่อสาธารณะและต้องรักษาเครดิตของผู้ให้.
        </p>
    </div>
    <?php
}

// Add watermark to newly uploaded images
add_filter('wp_generate_attachment_metadata', 'add_watermark_to_image', 10, 2);

function add_watermark_to_image($metadata, $attachment_id) {
    $upload_dir = wp_upload_dir();

    $original_file_path = $upload_dir['basedir'] . '/' . $metadata['file'];
    apply_watermark($original_file_path);

    if (isset($metadata['sizes'])) {
        foreach ($metadata['sizes'] as $size) {
            $size_file_path = $upload_dir['basedir'] . '/' . dirname($metadata['file']) . '/' . $size['file'];
            apply_watermark($size_file_path);
        }
    }

    return $metadata;
}

// Apply watermark with dynamic settings
function apply_watermark($file_path) {
    $mime_type = mime_content_type($file_path);

    $watermark_text = get_option('watermark_text', 'Your Watermark Text');
    $font_size = get_option('watermark_font_size', 16);
    $color_hex = get_option('watermark_color', '#ffffff');

    list($r, $g, $b) = sscanf($color_hex, "#%02x%02x%02x");

    $image = null;

    switch ($mime_type) {
        case 'image/jpeg':
            $image = imagecreatefromjpeg($file_path);
            break;
        case 'image/png':
            $image = imagecreatefrompng($file_path);
            break;
        case 'image/webp':
            $image = imagecreatefromwebp($file_path);
            break;
        default:
            return;
    }

    if ($image) {
        $font_angle = 0;
        $font_file = plugin_dir_path(__FILE__) . 'arial.ttf';
        $text_color = imagecolorallocate($image, $r, $g, $b);

        $image_width = imagesx($image);
        $image_height = imagesy($image);

        $text_box = imagettfbbox($font_size, $font_angle, $font_file, $watermark_text);
        $text_width = $text_box[2] - $text_box[0];
        $text_height = $text_box[1] - $text_box[7];

        $x_position = ($image_width - $text_width) / 2;
        $y_position = ($image_height + $text_height) / 2;

        imagettftext($image, $font_size, $font_angle, $x_position, $y_position, $text_color, $font_file, $watermark_text);

        switch ($mime_type) {
            case 'image/jpeg':
                imagejpeg($image, $file_path);
                break;
            case 'image/png':
                imagepng($image, $file_path);
                break;
            case 'image/webp':
                imagewebp($image, $file_path);
                break;
        }

        imagedestroy($image);
    }
}

// Apply watermark to old images
function watermark_bulk_apply_page() {
    if (isset($_POST['apply_watermark'])) {
        $attachments = get_posts([
            'post_type' => 'attachment',
            'posts_per_page' => -1,
        ]);

        foreach ($attachments as $attachment) {
            $file_path = get_attached_file($attachment->ID);
            apply_watermark($file_path);

            $metadata = wp_get_attachment_metadata($attachment->ID);
            $upload_dir = wp_upload_dir();

            if (isset($metadata['sizes'])) {
                foreach ($metadata['sizes'] as $size) {
                    $size_file_path = $upload_dir['basedir'] . '/' . dirname($metadata['file']) . '/' . $size['file'];
                    apply_watermark($size_file_path);
                }
            }
        }

        echo '<div class="updated"><p>Watermark applied to all images.</p></div>';
    }

    ?>
    <div class="wrap">
        <h1>Apply Watermark to Old Images</h1>
        <form method="POST">
            <p>Click the button below to apply the current watermark to all existing images.</p>
            <?php submit_button('Apply Watermark', 'primary', 'apply_watermark'); ?>
        </form>
    </div>
    <?php
}
?>
